@@ -11,7 +11,7 @@ use warnings;
 
 no warnings qw( uninitialized once );
 
-our $VERSION = '1.862';
+our $VERSION = '1.863';
 
 use threads;
 use threads::shared;
@@ -252,13 +252,18 @@ sub send2 {
    }
 
    local $\ = undef if (defined $\);
+   local $MCE::Signal::SIG;
 
    {
+      local $MCE::Signal::IPC = 1;
       CORE::lock $self->{cw_mutex};
+
       MCE::Util::_sock_ready_w( $self->{c_sock} ) if $is_MSWin32;
       print { $self->{c_sock} } pack('i', length $data), $data;
    }
 
+   CORE::kill($MCE::Signal::SIG, $$) if $MCE::Signal::SIG;
+
    return 1;
 }
 
@@ -336,7 +341,7 @@ MCE::Channel::Threads - Channel for producer(s) and many consumers
 
 =head1 VERSION
 
-This document describes MCE::Channel::Threads version 1.862
+This document describes MCE::Channel::Threads version 1.863
 
 =head1 DESCRIPTION
 
