@@ -11,23 +11,20 @@ use warnings;
 
 no warnings qw( threads recursion uninitialized once );
 
-our $VERSION = '1.862';
+our $VERSION = '1.863';
 
 use base 'MCE::Mutex';
 use Scalar::Util qw(refaddr weaken);
 use MCE::Util ();
 
-my $has_threads = $INC{'threads.pm'} ? 1 : 0;
-my $tid = $has_threads ? threads->tid()  : 0;
-
-my @MUTEX;
+my $tid = $INC{'threads.pm'} ? threads->tid() : 0;
 
 sub CLONE {
-    $tid = threads->tid() if $has_threads;
+    $tid = threads->tid() if $INC{'threads.pm'};
 }
 
 sub DESTROY {
-    my ($pid, $obj) = ($has_threads ? $$ .'.'. $tid : $$, @_);
+    my ($pid, $obj) = ($tid ? $$ .'.'. $tid : $$, @_);
 
     syswrite($obj->{_w_sock}, '0'), $obj->{$pid    } = 0 if $obj->{$pid    };
     syswrite($obj->{_r_sock}, '0'), $obj->{$pid.'b'} = 0 if $obj->{$pid.'b'};
@@ -38,27 +35,27 @@ sub DESTROY {
         ($^O eq 'MSWin32' && $obj->{impl} eq 'Channel')
             ? MCE::Util::_destroy_pipes($obj, qw(_w_sock _r_sock))
             : MCE::Util::_destroy_socks($obj, qw(_w_sock _r_sock));
-
-        if ( ! $has_threads ) {
-            @MUTEX = map { refaddr($_) == $addr ? () : $_ } @MUTEX;
-        }
     }
 
     return;
 }
 
-sub _destroy {
-    # Called by { MCE, MCE::Child, and MCE::Hobo }::_exit.
-    # This must iterate a copy.
+my @mutex;
 
-    if ( @MUTEX ) {
-        local $_;
-        &DESTROY($_) for @{[ @MUTEX ]};
+sub _destroy {
+    my $pid = $tid ? $$ .'.'. $tid : $$;
+
+    # Called by { MCE, MCE::Child, and MCE::Hobo }::_exit
+    for my $i ( 0 .. @mutex - 1 ) {
+        syswrite($mutex[$i]->{_w_sock}, '0'), $mutex[$i]->{$pid} = 0
+            if ( $mutex[$i]->{$pid} );
+        syswrite($mutex[$i]->{_r_sock}, '0'), $mutex[$i]->{$pid.'b'} = 0
+            if ( $mutex[$i]->{$pid.'b'} );
     }
 }
 
-sub _save_for_global_destruction {
-    push(@MUTEX, $_[0]), weaken $MUTEX[-1] unless $has_threads;
+sub _save_for_global_cleanup {
+    push(@mutex, $_[0]), weaken($mutex[-1]);
 }
 
 ###############################################################################
@@ -69,7 +66,7 @@ sub _save_for_global_destruction {
 
 sub new {
     my ($class, %obj) = (@_, impl => 'Channel');
-    $obj{'_init_pid'} = $has_threads ? $$ .'.'. $tid : $$;
+    $obj{'_init_pid'} = $tid ? $$ .'.'. $tid : $$;
 
     ($^O eq 'MSWin32')
         ? MCE::Util::_pipe_pair(\%obj, qw(_r_sock _w_sock))
@@ -80,14 +77,14 @@ sub new {
     bless \%obj, $class;
 
     if ( caller !~ /^MCE:?/ || caller(1) !~ /^MCE:?/ ) {
-        MCE::Mutex::Channel::_save_for_global_destruction(\%obj);
+        MCE::Mutex::Channel::_save_for_global_cleanup(\%obj);
     }
 
     return \%obj;
 }
 
 sub lock {
-    my ($pid, $obj) = ($has_threads ? $$ .'.'. $tid : $$, @_);
+    my ($pid, $obj) = ($tid ? $$ .'.'. $tid : $$, @_);
 
     MCE::Util::_sysread($obj->{_r_sock}, my($b), 1), $obj->{ $pid } = 1
         unless $obj->{ $pid };
@@ -99,7 +96,7 @@ sub lock {
 *lock_shared    = \&lock;
 
 sub unlock {
-    my ($pid, $obj) = ($has_threads ? $$ .'.'. $tid : $$, @_);
+    my ($pid, $obj) = ($tid ? $$ .'.'. $tid : $$, @_);
 
     syswrite($obj->{_w_sock}, '0'), $obj->{ $pid } = 0
         if $obj->{ $pid };
@@ -108,9 +105,7 @@ sub unlock {
 }
 
 sub synchronize {
-    my ($pid, $obj, $code, @ret) = (
-        $has_threads ? $$ .'.'. $tid : $$, shift, shift
-    );
+    my ($pid, $obj, $code, @ret) = ($tid ? $$ .'.'. $tid : $$, shift, shift);
     return unless ref($code) eq 'CODE';
 
     # lock, run, unlock - inlined for performance
@@ -144,7 +139,7 @@ MCE::Mutex::Channel - Mutex locking via a pipe or socket
 
 =head1 VERSION
 
-This document describes MCE::Mutex::Channel version 1.862
+This document describes MCE::Mutex::Channel version 1.863
 
 =head1 DESCRIPTION
 
